package com.loafle.overflow.module.generator.service;

import com.loafle.overflow.commons.utils.StringConvertor;
import com.loafle.overflow.crawler.config.Config;
import com.loafle.overflow.crawler.config.Connection;
import com.loafle.overflow.crawler.config.Crawler;
import com.loafle.overflow.crawler.config.Schedule;
import com.loafle.overflow.module.auth.model.AuthCrawler;
import com.loafle.overflow.module.auth.service.AuthCrawlerService;
import com.loafle.overflow.module.infra.model.Infra;
import com.loafle.overflow.module.infra.model.InfraHost;
import com.loafle.overflow.module.meta.model.MetaSensorItemKey;
import com.loafle.overflow.module.meta.type.MetaCrawlerEnum;
import com.loafle.overflow.module.sensor.model.Sensor;
import com.loafle.overflow.module.sensor.model.SensorItem;
import org.codehaus.jackson.map.ObjectMapper;
import org.codehaus.jackson.type.TypeReference;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.io.IOException;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * Created by snoop on 17. 9. 6.
 */
@Service("InfraHostGenerator")
public class InfraHostGenerator {

    @Autowired
    private GenerateUtil generateUtil;

    @Autowired
    private InfraHostWMIGenerator infraHostWMIGenerator;

    @Autowired
    private AuthCrawlerService authCrawlerService;

    public String process(Sensor dbSensor, List<SensorItem> sensorItems, Infra infra) throws IOException {

        InfraHost infraHost = (InfraHost)infra;

        Config config = new Config();
        config.setId(String.valueOf(dbSensor.getId()));

        com.loafle.overflow.crawler.config.Target target = this.createTarget(infraHost, dbSensor);

        if(target == null) {
            return null;
        }

        config.setTarget(target);

        // FIXME: Interval
        Schedule schedule = new Schedule();
        schedule.setInterval("5");
        config.setSchedule(schedule);

        Crawler crawler = this.generateUtil.getCrawler(dbSensor.getCrawler());
        config.setCrawler(crawler);

        Map<Integer, MetaSensorItemKey> keyMap = this.generateUtil.initMappingMap(dbSensor.getCrawler());

        if(dbSensor.getCrawler().getId() == MetaCrawlerEnum.WMI_CRAWLER.getValue()) {
            this.infraHostWMIGenerator.process(sensorItems, keyMap, dbSensor, config);
        }

        ObjectMapper objectMapper = new ObjectMapper();
        return objectMapper.writeValueAsString(config);
    }

    private com.loafle.overflow.crawler.config.Target createTarget(InfraHost infraHost, Sensor dbSensor) throws IOException {

        AuthCrawler authCrawler = this.authCrawlerService.readAuth(dbSensor.getCrawler(), dbSensor.getTarget());

        if(authCrawler == null) {
            return null;
        }

        com.loafle.overflow.crawler.config.Target target = new com.loafle.overflow.crawler.config.Target();
        Connection connection = new Connection();
        connection.setIp(StringConvertor.intToIp(infraHost.getIp()));

        HashMap<String,String> optionMap;
        optionMap = new ObjectMapper().readValue(authCrawler.getAuthJson(), new TypeReference<HashMap<String,String>>() {});

        if(dbSensor.getCrawler().getId() == MetaCrawlerEnum.WMI_CRAWLER.getValue()) {
            connection.setPort("135");
            connection.setPortType("tcp");
            connection.setSsl(false);

            target.setConnection(connection);

            Map<String, Object> auth = new HashMap<>();
            auth.put("id", optionMap.get("ID"));
            auth.put("pw", optionMap.get("PassWord"));

            target.setAuth(auth);

        }

        return target;
    }
}
