package com.loafle.overflow.module.member.service;

import com.loafle.overflow.commons.model.SessionMetadata;
import com.loafle.overflow.commons.utils.EmailSender;
import com.loafle.overflow.module.email.service.EmailAuthService;
import com.loafle.overflow.module.member.dao.MemberDAO;
import com.loafle.overflow.module.member.exception.*;
import com.loafle.overflow.module.member.model.Member;
import com.loafle.overflow.module.meta.model.MetaMemberStatus;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.mail.MailException;
import org.springframework.security.crypto.bcrypt.BCryptPasswordEncoder;
import org.springframework.stereotype.Service;

import java.io.UnsupportedEncodingException;
import java.net.URLDecoder;
import java.net.URLEncoder;

/**
 * Created by geek on 17. 6. 28.
 */
@Service("MemberService")
public class MemberService {

    @Autowired
    private MemberDAO memberDAO;

    @Autowired
    private EmailAuthService emailAuthService;

    @Autowired
    private EmailSender emailSender;

    private BCryptPasswordEncoder passwordEncoder = new BCryptPasswordEncoder();

    public Member signin(String signinId, String signinPw) throws SignInIdNotExistException, EmailNotConfirmedException, SignInPwNotMatchException {
        Member m = this.memberDAO.findByEmail(signinId);

        if ( null == m ) {
            throw new SignInIdNotExistException();
        }

        if ( m.getStatus().getId() == 1 ) {
            throw new EmailNotConfirmedException();
        }

        Boolean match = passwordEncoder.matches(signinPw, m.getPw());
        if(!match) {
            throw new SignInPwNotMatchException();
        }

        return m;
    }

    public Member signup(Member member, String pw) throws UnsupportedEncodingException {

        Member isMember = this.memberDAO.findByEmail(member.getEmail());

        if (null != isMember && isMember.getId() > 0) {
            throw new JoinedEmailException();
        }

        member.setPw(passwordEncoder.encode(pw));

        if (member.getStatus() == null) {
            member.setStatus(new MetaMemberStatus((short) 1));
        }

        Member resMember = this.memberDAO.save(member);

        try {
            this.emailAuthService.sendEmailByMember(member.getId(), member.getEmail());
        } catch (MailException e) {
            // Todo ReSend Mail
            e.printStackTrace();
        }


        return resMember;
    }

    public Member sendEmailForPassword(String email) throws UnsupportedEncodingException {
        Member member = this.memberDAO.findByEmail(email);

        if (null == member) {
            throw new SignInIdNotExistException();
        }
        String en = this.emailSender.encrypt(email);
        String encode = URLEncoder.encode(en, "UTF-8");

        try {
            this.emailSender.sendSimpleEmail(email,"Password Reset Email", "http://127.0.0.1:9091/#/account/reset_password?key="+ encode +"\r\nConfirm Email");
        } catch (MailException e) {
            // Todo ReSend Mail
            e.printStackTrace();
        }


        return member;
    }

    public Member resetPassword(String encodeEmail, String pw) throws UnsupportedEncodingException {
        String deStr = URLDecoder.decode(encodeEmail, "UTF-8");

        String deEmail = this.emailSender.decrypt(deStr);
        Member member = this.memberDAO.findByEmail(deEmail);

        if (null == member) {
            throw new SignInIdNotExistException();
        }

        member.setPw(passwordEncoder.encode(pw));

        return this.memberDAO.save(member);
    }

    public void signout(Member member) {
        // Todo websocket session remove
    }

    public Member modify(Member member, String pw) {
        String email = SessionMetadata.getEmail();

        Member preMember = this.memberDAO.findByEmail(member.getEmail());

        if (null != pw && !pw.equals("")) {
            Boolean match = passwordEncoder.matches(member.getPw(), preMember.getPw());
            if(!match) {
                member.setPw(passwordEncoder.encode(pw));
            }
        } else {
            member.setPw(preMember.getPw());
        }

        if (member.getStatus() == null || member.getStatus().getId() < 0) {
            member.getStatus().setId(preMember.getStatus().getId());
        }
        return this.memberDAO.save(member);
    }

    public Member confirmPw(String signinId, String signinPw) {
        Member preMember = this.memberDAO.findByEmail(signinId);

        String encodePw = passwordEncoder.encode(signinPw);
        Boolean match = passwordEncoder.matches(encodePw, preMember.getPw());

        if (!match) {
            throw new SignInPwNotMatchException();
        }

        return preMember;
    }

    public Member forgotPassword(String signinId, String newPw) {
        Member preMember = this.memberDAO.findByEmail(signinId);

        if (null == preMember) {
            throw new SignInIdNotExistException();
        }

        preMember.setPw(this.passwordEncoder.encode(newPw));
        this.memberDAO.save(preMember);

        return preMember;
    }

    public Member read(long memberId) {
        if (memberId <= 0) {
            throw new SignInIdNotExistException();
        }

        Member resMember = this.memberDAO.findOne(memberId);


        return resMember;
    }

    public void withdrawal(Member member) {
        String email = SessionMetadata.getEmail();

        // Todo DB delete?
    }

    public void emailConfirm(Member member) {

    }
}
