package com.loafle.overflow.module.noauthprobe.service;

import com.loafle.overflow.module.domain.model.Domain;
import com.loafle.overflow.module.member.model.Member;
import com.loafle.overflow.module.meta.model.MetaNoAuthProbeStatus;
import com.loafle.overflow.module.meta.model.MetaProbeStatus;
import com.loafle.overflow.module.noauthprobe.dao.NoAuthProbeDAO;
import com.loafle.overflow.module.noauthprobe.model.NoAuthProbe;
import com.loafle.overflow.module.probe.model.Probe;
import com.loafle.overflow.module.probe.service.ProbeService;
import org.codehaus.jackson.map.ObjectMapper;
import org.codehaus.jackson.type.TypeReference;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.crypto.bcrypt.BCryptPasswordEncoder;
import org.springframework.stereotype.Service;

import java.io.IOException;
import java.util.*;

/**
 * Created by snoop on 17. 6. 28.
 */
@Service("NoAuthProbeService")
public class NoAuthProbeService {

    @Autowired
    private NoAuthProbeDAO noAuthProbeDAO;

    @Autowired
    private ProbeService probeService;

    @Autowired
    private ObjectMapper objectMapper;

    public NoAuthProbe regist(NoAuthProbe noAuthProbe) {

        noAuthProbe.setTempProbeKey(UUID.randomUUID().toString());
        noAuthProbe.setStatus(new MetaNoAuthProbeStatus((short)3));

        return this.noAuthProbeDAO.save(noAuthProbe);
    }


    public List<NoAuthProbe> readAllByDomain(Domain domain) {

        return this.noAuthProbeDAO.findAllByDomain(domain);
    }

    public NoAuthProbe read(long id) {
        return this.noAuthProbeDAO.findOne(id);
    }

    public List<NoAuthProbe> acceptNoAuthProbes(List<NoAuthProbe> noAuthProbes) throws IOException {

        // Todo domain injection & member injection
        List<Probe> probes = new ArrayList<>();
        Probe probe = null;

        Map<String, Object> objMap = null;
        Map<String, String> hostMap = null;
        Map<String, String> netMap = null;

        for (NoAuthProbe noAuth : noAuthProbes) {
            BCryptPasswordEncoder passwordEncoder = new BCryptPasswordEncoder();
            String encryptKey = passwordEncoder.encode(UUID.randomUUID().toString());

            probe = new Probe();
            probe.setEncryptionKey(encryptKey);
            probe.setProbeKey(noAuth.getApiKey());
            probe.setDomain(new Domain(1));
            probe.setAuthorizeMember(new Member(1));
            probe.setStatus(new MetaProbeStatus((short)1));
            probe.setTargetCount(0);
            probe.setSensorCount(0);

            objMap = this.objectMapper.readValue(noAuth.getDescription(), new TypeReference<HashMap<String,Object>>() {});
            hostMap = (Map<String, String>)objMap.get("host");
            netMap = (Map<String, String>)objMap.get("network");

//            String dispName = noAuth.getHostName().isEmpty() ?
//                    StringConvertor.intToIp(noAuth.getIpAddress()) : noAuth.getHostName();
            String dispName = hostMap.get("name");
            dispName += " probe";
            probe.setDisplayName(dispName);

            String addrStr = netMap.get("address");

            String[] addrArr = addrStr.split("\\|");

            System.out.println(addrArr);

            probe.setCidr(addrArr[0]);

            probes.add(probe);
            noAuth.setStatus(new MetaNoAuthProbeStatus((short) 1));
        }

        this.probeService.regist(probes);
        this.noAuthProbeDAO.save(noAuthProbes);
        return this.readAllByDomain(noAuthProbes.get(0).getDomain());
    }

    public List<NoAuthProbe> denyNoauthProbes(List<NoAuthProbe> noAuthProbes) {
        for (NoAuthProbe noAuth : noAuthProbes) noAuth.setStatus(new MetaNoAuthProbeStatus((short) 2));
        this.noAuthProbeDAO.save(noAuthProbes);
        return this.readAllByDomain(noAuthProbes.get(0).getDomain());
    }

    public NoAuthProbe readByTempKey(String tempKey) {
        return this.noAuthProbeDAO.findByTempProbeKey(tempKey);
    }
}
